@@ -1,16 +1,16 @@
 from armstrong.dev.tests.utils.backports import override_settings
-import datetime
 from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.test.client import Client
+from functools import wraps
 import fudge
 import os
 import random
 from unittest import expectedFailure
 
-from ._utils import no_initial_patched_objects
 from ._utils import TestCase
 
+from .. import constants
 from .. import forms
 from .. import models
 from .. import views
@@ -106,7 +106,6 @@ class BaseDonationFormViewTestCase(TestCase):
 
 # TODO: move to armstrong.dev
 def get_response(func):
-    from functools import wraps
     @wraps(func)
     def inner(self):
         func(self, self.get_response())
@@ -164,6 +163,23 @@ class DonationFormViewPostTestCase(BaseDonationFormViewTestCase):
         donation = models.Donation.objects.get(donor=donor)
         self.assertEqual(donation.amount, random_amount)
 
+    def test_uses_promo_code_if_available(self):
+        promo_code = self.random_discount
+        donor_name = self.random_donor_name
+        random_amount = self. random_amount
+        data = self.get_base_random_data(name=donor_name, amount=random_amount,
+                promo_code=promo_code.code)
+        data.update(self.get_data_as_formset())
+
+        self.client.post(self.url, data)
+        donor = models.Donor.objects.get(name=donor_name)
+        donation = models.Donation.objects.get(donor=donor)
+        self.assertEqual(promo_code, donation.code)
+
+        d = fudge.Fake().has_attr(amount=random_amount)
+        self.assertEqual("%d" % promo_code.calculate(d),
+                "%d" % donation.amount)
+
     def test_saves_address_if_present(self):
         donor_name = self.random_donor_name
         address_kwargs = self.random_address_kwargs
@@ -187,6 +203,7 @@ class DonationFormViewPostTestCase(BaseDonationFormViewTestCase):
         ])
         data = self.get_base_random_data(name=donor_name)
         data.update(address_formset)
+        del data[constants.MAILING_SAME_AS_BILLING]
 
         self.assertEqual(0, len(models.DonorAddress.objects.all()),
             msg="sanity check")
@@ -226,6 +243,17 @@ class DonationFormViewPostTestCase(BaseDonationFormViewTestCase):
         donor = models.Donor.objects.get(name=data["name"])
         self.assertEqual(donor.address, donor.mailing_address)
 
+    def test_same_as_billing_overrides_second_address(self):
+        data = self.random_post_data
+        data.update(self.get_data_as_formset([
+            self.random_address_kwargs,
+            self.random_address_kwargs,
+        ]))
+        data["mailing_same_as_billing"] = u"1"
+        self.client.post(self.url, data)
+        donor = models.Donor.objects.get(name=data["name"])
+        self.assertEqual(donor.address, donor.mailing_address)
+
     def test_redirects_to_success_url_after_successful_save(self):
         data = self.random_post_data
         response = self.client.post(self.url, data)
